program pswitch;

{$R *.res}

uses
  Windows, Messages, IniFiles, SysUtils, Registry;

var
  Ini: TIniFile;
  Profile: Integer;
  Time: Integer;
  LastChange: Integer;
  acpower: Boolean;
  SysPowerStatus: TSystemPowerStatus;

function PostAtkOsdMessage(ImageSet, ImageIndex: Word): Boolean;
var
  hWnd: LongWord;
  wParam: Integer;
begin
  Result := False;
  hWnd := FindWindow('ATKOSD', 'ATKOSD');
  if hWnd = 0 then Exit;

  if ImageSet in [92, 108] then
    wParam := ImageIndex shl 16 + ImageSet
  else
    wParam := ImageIndex;

  if not PostMessage(hWnd, WM_COMMAND, wParam, 0) then Exit;
  Result := True;
end;

function PostRMClockMessage(ProfileIndex: Word): Boolean;
var
  hWnd: LongWord;
  wParam: Integer;
begin
  Result := False;
  if not (ProfileIndex in [0..3]) then Exit;
  hWnd := FindWindow('#32770','RightMark CPU Clock Utility');
  if hWnd = 0 then Exit;

  wParam := ProfileIndex + $800E;  // RMClock 2.1
  if not PostMessage(hWnd, WM_COMMAND, wParam, 0) then Exit;
  Result := True;
end;

function SetATIProfile(ProfileName: string): Boolean;
var
  Path, CmdLine: string;
  StartInfo: TStartupInfo;
  ProcInfo: TProcessInformation;
begin
  Result := False;

  with TRegistry.Create do begin
    RootKey := HKEY_LOCAL_MACHINE;
    OpenKey('Software\ATI\ACE', False);
    Path := ReadString('Path');
    Free;
  end;
  if Path = '' then Path := 'c:\Program Files\ATI Technologies\ATI.ACE\';
  CmdLine := Format('%sCLI.exe Load profilename="%s"', [Path, ProfileName]);

  FillChar(StartInfo,SizeOf(TStartupInfo),#0);
  FillChar(ProcInfo,SizeOf(TProcessInformation),#0);
  StartInfo.cb := SizeOf(TStartupInfo);

  if CreateProcess(nil, PChar(CmdLine), nil, nil, False,
    CREATE_NEW_PROCESS_GROUP + NORMAL_PRIORITY_CLASS, nil, nil,
    StartInfo, ProcInfo) then
  begin
    WaitForSingleObject(ProcInfo.hProcess, 1000);
    Result := True;
  end;
end;



procedure ActivateProfile;
var
  OK: Boolean;
begin
  OK := True;
  if Ini.ReadBool('Main', 'RMClock', False) then
    OK := PostRMClockMessage(Ini.ReadInteger(IntToStr(Profile), 'RMClockProfile', 0));
  if OK and Ini.ReadBool('Main', 'AtiCatalyst', False) then
    SetATIProfile(Ini.ReadString(IntToStr(Profile), 'AtiCatalystProfile', ''));
end;

procedure ShowOsd;
begin
  if Ini.ReadBool('Main', 'ShowAtkOsd', False) then
    PostAtkOsdMessage(Ini.ReadInteger(IntToStr(Profile), 'AtkOsdImageSet', 0),
                      Ini.ReadInteger(IntToStr(Profile), 'AtkOsdImageIndex', 0));
end;



begin
  Ini := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'pswitch.ini');
  try
    try

      // Zjistime jestli je NB napajen ze site, nebo z baterie
      GetSystemPowerStatus(SysPowerStatus);
      acpower := SysPowerStatus.ACLineStatus = 1;

      // pouze zobrazi osd
      if (LowerCase(ParamStr(1)) = 'osd') and (ParamCount = 3) then begin
        PostAtkOsdMessage(StrToInt(ParamStr(2)), StrToInt(ParamStr(3)));
        Exit;

      // reaktivuje aktualni profil
      end else if (LowerCase(ParamStr(1)) = 'current') and (ParamCount = 1) then begin
        if acpower then Profile := Ini.ReadInteger('System', 'CurrentProfileAC', 0)
                   else Profile := Ini.ReadInteger('System', 'CurrentProfileBatt', 0);
        ShowOsd;
        ActivateProfile;

      // aktivuje nasledujici profil
      end else if (LowerCase(ParamStr(1)) = 'next') and (ParamCount = 1) then begin
        if acpower then Profile := Ini.ReadInteger('System', 'CurrentProfileAC', 0)
                   else Profile := Ini.ReadInteger('System', 'CurrentProfileBatt', 0);

        LastChange := Ini.ReadInteger('System', 'LastChange', 0);
        Time := GetTickCount;
        Ini.WriteInteger('System', 'LastChange', Time);

        if LastChange > Time then LastChange := 0;

        if (Time - Ini.ReadInteger('Main', 'Delay', 2200)) < LastChange then begin
          Inc(Profile);
          if Profile >= Ini.ReadInteger('Main', 'ProfileCount', 1) then Profile := 0;

          if acpower then Ini.WriteInteger('System', 'CurrentProfileAC', Profile)
                     else Ini.WriteInteger('System', 'CurrentProfileBatt', Profile);
        end;

        ShowOsd;
        ActivateProfile;

      // aktivuje profil zadany parametrem
      end else if (LowerCase(ParamStr(1)) = 'profile') and (ParamCount = 2) then begin
        Profile := StrToInt(ParamStr(1));
        if acpower then Ini.WriteInteger('System', 'CurrentProfileAC', Profile)
                   else Ini.WriteInteger('System', 'CurrentProfileBatt', Profile);
        ShowOsd;
        ActivateProfile;

      // jinak zobrazi napovedu, stejne jako pri chybe
      end else Abort;
    except
      MessageBox(0, 'Usage:'#13#10#13#10 +
                    'pswitch current ... reactivate current profile'#13#10 +
                    'pswitch next ... activate next profile'#13#10 +
                    'pswitch profile profile_index ... activate specified profile'#13#10 +
                    'pswitch osd image_set image_index ... shows specified AtkOsd',
                    nil, MB_OK + MB_ICONERROR);
      Exit;
    end;
  finally
    Ini.Free;
  end;
end.
